# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: msg_sign_data.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import gogo_pb2 as gogo__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='msg_sign_data.proto',
  package='cosmos.crypto.secp256k1',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x13msg_sign_data.proto\x12\x17\x63osmos.crypto.secp256k1\x1a\ngogo.proto\"r\n\x0bMsgSignData\x12K\n\x06Signer\x18\x01 \x01(\x0c\x42;\xea\xde\x1f\x06signer\xfa\xde\x1f-github.com/cosmos/cosmos-sdk/types.AccAddress\x12\x16\n\x04\x44\x61ta\x18\x02 \x01(\x0c\x42\x08\xea\xde\x1f\x04\x64\x61tab\x06proto3'
  ,
  dependencies=[gogo__pb2.DESCRIPTOR,])




_MSGSIGNDATA = _descriptor.Descriptor(
  name='MsgSignData',
  full_name='cosmos.crypto.secp256k1.MsgSignData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='Signer', full_name='cosmos.crypto.secp256k1.MsgSignData.Signer', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\352\336\037\006signer\372\336\037-github.com/cosmos/cosmos-sdk/types.AccAddress', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='Data', full_name='cosmos.crypto.secp256k1.MsgSignData.Data', index=1,
      number=2, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\352\336\037\004data', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=60,
  serialized_end=174,
)

DESCRIPTOR.message_types_by_name['MsgSignData'] = _MSGSIGNDATA
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

MsgSignData = _reflection.GeneratedProtocolMessageType('MsgSignData', (_message.Message,), {
  'DESCRIPTOR' : _MSGSIGNDATA,
  '__module__' : 'msg_sign_data_pb2'
  # @@protoc_insertion_point(class_scope:cosmos.crypto.secp256k1.MsgSignData)
  })
_sym_db.RegisterMessage(MsgSignData)


_MSGSIGNDATA.fields_by_name['Signer']._options = None
_MSGSIGNDATA.fields_by_name['Data']._options = None
# @@protoc_insertion_point(module_scope)



import base64
import hashlib
import json
import ecdsa

from cyberpy._wallet import privkey_to_address, privkey_to_pubkey, address_to_address



class Message:
    """A cyber message. It's possible to create transaction-like
    message and sign it. ADR 036: Arbitrary Message Signature Specification
    """

    def __init__(
        self,
        *,
        privkey: bytes,
    ) -> None:
        self._privkey = privkey
        self._msgs: list = []

    def add_message(self, signing_message: str, signer_prefix: str = 'bostrom') -> None:
        message_b64 = base64.b64encode(str.encode(signing_message)).decode("utf-8")
        msg = {
            "type": "sign/MsgSignData",
            "value": {
                "data": message_b64,
                "signer": address_to_address(privkey_to_address(self._privkey), signer_prefix)
            },
        }
        self._msgs.append(msg)

    def add_message_custom(self, signing_message_list: list, signer_prefix: str = 'bostrom') -> None:
        signing_message = ':'.join(signing_message_list)
        message_b64 = base64.b64encode(str.encode(signing_message)).decode("utf-8")
        msg = {
            "type": "sign/MsgSignData",
            "value": {
                "data": message_b64,
                "signer": address_to_address(privkey_to_address(self._privkey), signer_prefix)
            },
        }
        self._msgs.append(msg)

    def get_signed_message(self) -> dict:
        pubkey = privkey_to_pubkey(self._privkey)
        base64_pubkey = base64.b64encode(pubkey).decode("utf-8")
        return {
            "pub_key": base64_pubkey,
            "signature": self._sign()
        }

    def _sign(self) -> str:
        message_str = json.dumps(self._get_sign_message(), separators=(",", ":"), sort_keys=True)
        message_bytes = message_str.encode("utf-8")

        privkey = ecdsa.SigningKey.from_string(self._privkey, curve=ecdsa.SECP256k1)
        signature_compact = privkey.sign_deterministic(
            message_bytes, hashfunc=hashlib.sha256, sigencode=ecdsa.util.sigencode_string_canonize
        )

        signature_base64_str = base64.b64encode(signature_compact).decode("utf-8")
        return signature_base64_str

    def _get_sign_message(self) -> dict:
        return {
            "chain_id": "",
            "account_number": "0",
            "fee": {
                "gas": "0",
                "amount": [],
            },
            "memo": '',
            "sequence": "0",
            "msgs": self._msgs,
        }